import os
import codecs
import pip
from setuptools import setup, find_packages

import riscof

# Base directory of package
here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


#Long Description
with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(name="riscof",
      version=riscof.__version__,
      description="Risc-V Compliance Framework",
      long_description=long_description,
      classifiers=[
          "Programming Language :: Python :: 3.7",
          "License :: OSI Approved :: BSD License",
          "Development Status :: 4 - Beta"
      ],
      url='https://gitlab.com/incoresemi/riscof',
      author='InCore Semiconductors Pvt. Ltd.',
      author_email='neelgala@incoresemi.com',
      license='BSD-3-Clause',
      packages=find_packages(),
      package_dir={'riscof': 'riscof/'},
      package_data={
          'riscof': [
              'suite/env/*', 'suite/rv32i_m/I/*.S', 'suite/rv32i_m/M/*.S',
              'framework/database.yaml', 'Templates/report.html',
              'Templates/style.css', 'Templates/setup/*',
              'Templates/setup/env/*'
          ]
      },
      install_requires=[
          'Cerberus>=1.3.1', 'GitPython>=2.1.11', 'oyaml>=0.9', 'PyYAML>=5.1.1',
          'jinja2>=2.10.1', 'pytz>=2019.1', 'rifle==1.1.0'
      ],
      python_requires=">=3.7.0",
      entry_points={
          "console_scripts": ["riscof=riscof.main:execute"],
      })



import logging
import importlib
from datetime import datetime
import os
import pytz
import shutil
import configparser

from jinja2 import Template

import riscof
import rifle.checker as rifle
import riscof.framework.main as framework
import riscof.utils as utils
import riscof.constants as constants
from rifle.errors import ValidationError


def execute():
    '''
        Entry point for riscof. This function sets up the models and 
        calls the :py:mod:`rifle` and :py:mod:`framework` modules with 
        appropriate arguments.
    '''
    # Set up the parser
    parser = utils.riscof_cmdline_args()
    args = parser.parse_args()

    # Set up the logger
    utils.setup_logging(args.verbose)
    logger = logging.getLogger()
    logger.handlers = []
    ch = logging.StreamHandler()
    ch.setFormatter(utils.ColoredFormatter())
    logger.addHandler(ch)
    fh = logging.FileHandler('run.log', 'w')
    logger.addHandler(fh)

    if (args.run):
        config = configparser.ConfigParser()
        logger.info("Reading configuration.")
        try:
            config.read('config.ini')
        except FileNotFoundError as err:
            logger.error(err)
            return 1

        riscof_config = config['RISCOF']
        logger.info("Preparing Models")
        # Gathering Models
        dut_model = riscof_config['DUTPlugin']
        base_model = riscof_config['ReferencePlugin']
        logger.debug("Importing " + dut_model + " plugin")
        dut_plugin = importlib.import_module("riscof_" + dut_model)
        dut_class = getattr(dut_plugin, dut_model)
        if dut_model in config:
            dut = dut_class(name="DUT", config=config[dut_model])
        else:
            dut = dut_class(name="DUT")
        logger.debug("Importing " + base_model + " plugin")
        base_plugin = importlib.import_module("riscof_" + base_model)
        base_class = getattr(base_plugin, base_model)
        if base_model in config:
            base = base_class(name="Reference", config=config[base_model])
        else:
            base = base_class(name="Reference")

        #Run rifle on inputs
        isa_file = dut.isa_spec
        platform_file = dut.platform_spec

        work_dir = constants.work_dir
        #Creating work directory
        if not os.path.exists(work_dir):
            logger.debug('Creating new work directory: ' + work_dir)
            os.mkdir(work_dir)
        else:
            logger.debug('Removing old work directory: ' + work_dir)
            shutil.rmtree(work_dir)
            logger.debug('Creating new work directory: ' + work_dir)
            os.mkdir(work_dir)

        try:
            isa_file, platform_file = rifle.check_specs(isa_file, platform_file,
                                                        work_dir)
        except ValidationError as msg:
            logger.error(msg)
            return 1

        report_objects = {}
        report_objects['date'] = (datetime.now(
            pytz.timezone('GMT'))).strftime("%Y-%m-%d %H:%M GMT")
        report_objects['version'] = riscof.__version__
        report_objects['dut'] = (dut.__model__).replace("_", " ")
        report_objects['reference'] = (base.__model__).replace("_", " ")

        isa_specs = utils.load_yaml(isa_file)

        with open(isa_file, "r") as isafile:
            ispecs = isafile.read()

        with open(platform_file, "r") as platfile:
            platform_specs = platfile.read()

        report_objects['isa'] = isa_specs['ISA']
        report_objects['usv'] = isa_specs['User_Spec_Version']
        report_objects['psv'] = isa_specs['Privilege_Spec_Version']
        report_objects['isa_yaml'] = isa_file
        report_objects['platform_yaml'] = platform_file
        report_objects['isa_specs'] = ispecs
        report_objects['platform_specs'] = platform_specs

        report_objects['results'] = framework.run(dut, base, isa_file,
                                                  platform_file)

        report_objects['num_passed'] = 0
        report_objects['num_failed'] = 0

        for entry in report_objects['results']:
            if entry['res'] == "Passed":
                report_objects['num_passed'] += 1
            else:
                report_objects['num_failed'] += 1

        with open(constants.html_template, "r") as report_template:
            template = Template(report_template.read())

        output = template.render(report_objects)

        reportfile = os.path.join(constants.work_dir, "report.html")
        with open(reportfile, "w") as report:
            report.write(output)

        shutil.copyfile(constants.css,
                        os.path.join(constants.work_dir, "style.css"))

        try:
            import webbrowser
            webbrowser.open(reportfile)
        except:
            return 0
    elif (args.setup):
        logger.info("Setting up environment files.")
        try:
            logger.debug(
                "Copying sample directory containing environment files.")
            cwd = os.getcwd()
            src = os.path.join(constants.root, "Templates/setup/env/")
            dest = os.path.join(cwd, "env/")
            shutil.copytree(src, dest)
            logger.debug("Copying sample config file.")
            src = os.path.join(constants.root, "Templates/setup/config.ini")
            shutil.copy(src, cwd)
            return 0
        except FileExistsError as err:
            logger.error(err)
            return 1


if __name__ == "__main__":
    exit(execute())



import os

root = os.path.abspath(os.path.dirname(__file__))

suite = "suite/"
framework_db = os.path.join(root, "framework/database.yaml")
cwd = os.getcwd()
work_dir = os.path.join(cwd, "riscof_work/")
html_template = os.path.join(root, 'Templates/report.html')
css = os.path.join(root, 'Templates/style.css')
env = os.path.join(root, suite + "env/")



import random
import string
import logging
from abc import abstractmethod, ABC

logger = logging.getLogger(__name__)


class pluginTemplate(ABC):
    """
        Metaclass for plugins as supported by :py:mod:`ABC`.
    """
    __model__ = "Template"

    @abstractmethod
    def __init__(self, *args, **kwargs):
        """
            Constructor.
            
            :param name: (passed as kwarg) Name to be displayed in the logger.
            
            :param config: (passed as kwarg) The configuration for the plugin as specified in the conifig.ini file.

            :type config: dict

            :type name: str
        """
        self.name = kwargs.get(
            'name',
            ''.join(random.choices(string.ascii_uppercase + string.digits,
                                   k=10)))

    @abstractmethod
    def initialise(self, suite, workdir, env):
        """
            Initialise the plugin with neccessary parameters.
            
            :param suite: The name of the suite directory.This is used to replace the name of the file to create 
                directories in proper order.
            
            :param workdir: The absolute path to the work directory.

            :param env: The directory containing the header files for the tests.

            :type suite: str

            :type workdir: str
            
            :type env: str
        """
        logger.debug(self.name + "Initialise")
        pass

    @abstractmethod
    def build(self, isa_yaml, platform_yaml):
        """
            Build the model as per specifications specified by DUT.
            
            :param isa_yaml: Path to the checked isa specs yaml.
            
            :param platform_yaml: Path to the checked platform specs yaml.
            
            :type isa_yaml: str
            
            :type platform_yaml: str
        """
        logger.debug(self.name + "Build")
        pass

    @abstractmethod
    def runTests(self, testlist):
        """Use the model to run the tests and produce signatures. The signature files generated should be named-*self.name[:-1]+".signature"*.
            
            :return: The absolute path to the signature file generated.
            
            :param testlist: A dictionary of tests and other information about them(like macros,work_dir and isa).
            
            :type testlist: dict
        """
        logger.debug(self.name + "Test Run")
        pass

    def getname(self):
        return self._role + "-" + self.__model__ + ":"

    def setname(self, role):
        self._role = role

    name = property(getname, setname)
